#!/usr/bin/env python3
"""
Main steering script for evaluation
"""
from shutil import copyfile
import argparse
import os
import ROOT
import HelperModules.HelperFunctions as hf
from HelperModules.MessageHandler import WelcomeMessage
from HelperModules import Settings
from HelperModules.Directories import Directories
from PlotterModules.GANPlotHandler import GANPlotHandler
from PlotterModules.RegressionPlotHandler import RegressionPlotHandler
from PlotterModules.ClassificationPlotHandler import ClassificationPlotHandler

if __name__ == "__main__":
    ROOT.gErrorIgnoreLevel = ROOT.kWarning
    parser = argparse.ArgumentParser(formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument("-c", "--configfile", help="Config file", required=True)
    parser.add_argument("--plots", nargs="+", help="the dedicated plots to be produced, default is 'all'", default=["all"], choices=["all", "YieldsPlot", "StackPlots", "Stack2DPlots", "Separation1D", "TrainTestPlots", "NonnegativeWeightsComparison", "SoverBPlots", "SoverB2DPlots", "CorrelationPlots", "BinaryConfusionPlots", "ROCCurvePlots", "AUCSummaries", "ConfusionMatrix", "PermutationImportances"])
    args = parser.parse_args()
    WelcomeMessage("Evaluate")

    cfg_settings = Settings.Settings(args.configfile, option="all") # The settings object
    Dirs = Directories("%s"%cfg_settings.get_General().get("Job"))

    # Copy the config file to the job directory
    cfg_path, cfg_extension = os.path.splitext(args.configfile)
    copyfile(args.configfile, hf.ensure_trailing_slash(Dirs.ConfigDir()) +  os.path.basename(cfg_path) + "_evaluation_step" + cfg_extension)
    Model = cfg_settings.get_Model()
    ###################################################################################
    ############################### GAN Classification ################################
    ###################################################################################
    if Model.isClassificationWGAN() or Model.isClassificationACGAN():
        DH = GANPlotHandler(cfg_settings)
        DH.do_Plots()
    ##################################################################################
    ############################# DNN/BDT Classification #############################
    ##################################################################################
    elif Model.isClassificationDNN() or Model.isClassificationBDT():
        DH = ClassificationPlotHandler(cfg_settings)
        DH.do_Plots(plots=args.plots)
    #####################################################################################
    ################################## Reconstruction ###################################
    #####################################################################################
    elif Model.isReconstruction():
        DH = ClassificationPlotHandler(cfg_settings)
        DH.do_Plots()
    #####################################################################################
    #################################### Regression #####################################
    #####################################################################################
    elif Model.isRegressionDNN() or Model.isRegressionBDT():
        DH = RegressionPlotHandler(cfg_settings)
        DH.do_Plots()
